/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvSignalEnvelope.h"
#include "HvSignalLine.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"
#include "HvControlIf.h"
#include "HvMath.h"
#include "HvSignalVar.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_8amAoGkx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WkQqwl1X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BKYlTH9a_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SSBvbfDn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ZiJPsGxK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_T0QZKK7V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_h7EaIrWx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_mb8wb0t8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hyCdaPgF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RBtG0seG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fktslXGG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_jZ110CDV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_6LqdB7O2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_m57ZD1Ol_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_3140AAiA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_zwI321Ux_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_fYmtZ49j_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_GDeqVr0w_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_90Iquw6T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MDQlBi3b_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_qENA8Z8O_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Te7YVThZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_X6bFw1pw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_tasb83sb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_PkFq7ZbY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OIilZYpH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uJVeExHP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5RKYaCAP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fK18TvT2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0MVd4Dds_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YfN7300h_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_1n02p2kW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_vn0arwYt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_psRp1gC8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_ITnwE5dM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_ZFlILg2I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fP12XKvv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_7XJWIQd3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6hzBUnE1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_353RnPLB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0k71FiM3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Rq3YStuQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kpwTbxy7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CONMBRZH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_F4yALt5d_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_W6RMJqhr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_cayBIkVf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KkGSMhXE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_xOx2boso_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_g6cB7uRg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_kCp2E03c_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_tvICsOqj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_apwEVNB3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kLjQOg4h_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_q5kIJ0d7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_hN5bY9YJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_cukrF3E2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_5VaDAwuQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_cx1u53Ws_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_hSpOmZAO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_y4LfFMXY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_m6KlSflE_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_Tuo2TwXm;
  SignalEnvelope sEnv_GDeqVr0w;
  SignalEnvelope sEnv_90Iquw6T;
  SignalEnvelope sEnv_ITnwE5dM;
  SignalEnvelope sEnv_ZFlILg2I;
  SignalLine sLine_yZCXGpY1;
  SignalLine sLine_ecVxQ8f2;
  ControlBinop cBinop_8amAoGkx;
  ControlBinop cBinop_WkQqwl1X;
  ControlBinop cBinop_BKYlTH9a;
  ControlBinop cBinop_SSBvbfDn;
  ControlBinop cBinop_h7EaIrWx;
  ControlBinop cBinop_hyCdaPgF;
  ControlBinop cBinop_RBtG0seG;
  ControlBinop cBinop_fktslXGG;
  ControlVar cVar_jZ110CDV;
  ControlVar cVar_m57ZD1Ol;
  SignalVarf sVarf_9HfJoyjP;
  ControlVar cVar_3140AAiA;
  ControlIf cIf_zwI321Ux;
  ControlIf cIf_fYmtZ49j;
  ControlBinop cBinop_MDQlBi3b;
  ControlBinop cBinop_Te7YVThZ;
  ControlBinop cBinop_X6bFw1pw;
  ControlBinop cBinop_OIilZYpH;
  ControlBinop cBinop_5RKYaCAP;
  SignalVarf sVarf_hOFtApux;
  ControlVar cVar_1n02p2kW;
  ControlIf cIf_vn0arwYt;
  ControlIf cIf_psRp1gC8;
  ControlBinop cBinop_fP12XKvv;
  ControlBinop cBinop_6hzBUnE1;
  ControlBinop cBinop_353RnPLB;
  ControlBinop cBinop_kpwTbxy7;
  ControlBinop cBinop_F4yALt5d;
  ControlVar cVar_xOx2boso;
  ControlBinop cBinop_g6cB7uRg;
  SignalVarf sVarf_5YKzG1lR;
  SignalVarf sVarf_rGJWKqYI;
  SignalVarf sVarf_B3oe3gHm;
  SignalVarf sVarf_aXbnbIiU;
  ControlBinop cBinop_kLjQOg4h;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
